@@ -47,6 +47,7 @@
 #define MGMT_STATUS_INVALID_INDEX	0x11
 #define MGMT_STATUS_RFKILLED		0x12
 #define MGMT_STATUS_ALREADY_PAIRED	0x13
+#define MGMT_STATUS_PERMISSION_DENIED	0x14
 
 struct mgmt_hdr {
 	uint16_t opcode;
@@ -273,14 +274,10 @@ struct mgmt_cp_user_passkey_neg_reply {
 
 #define MGMT_OP_READ_LOCAL_OOB_DATA	0x0020
 struct mgmt_rp_read_local_oob_data {
-	uint8_t hash[16];
-	uint8_t randomizer[16];
-} __packed;
-struct mgmt_rp_read_local_oob_ext_data {
 	uint8_t hash192[16];
-	uint8_t randomizer192[16];
+	uint8_t rand192[16];
 	uint8_t hash256[16];
-	uint8_t randomizer256[16];
+	uint8_t rand256[16];
 } __packed;
 
 #define MGMT_OP_ADD_REMOTE_OOB_DATA	0x0021
@@ -456,7 +453,80 @@ struct mgmt_cp_start_service_discovery {
 	uint16_t uuid_count;
 	uint8_t uuids[0][16];
 } __packed;
-#define MGMT_START_SERVICE_DISCOVERY_SIZE	4
+
+#define MGMT_OP_READ_LOCAL_OOB_EXT_DATA	0x003B
+struct mgmt_cp_read_local_oob_ext_data {
+	uint8_t  type;
+} __packed;
+struct mgmt_rp_read_local_oob_ext_data {
+	uint8_t  type;
+	uint16_t eir_len;
+	uint8_t  eir[0];
+} __packed;
+
+#define MGMT_OP_READ_EXT_INDEX_LIST	0x003C
+struct mgmt_rp_read_ext_index_list {
+	uint16_t num_controllers;
+	struct {
+		uint16_t index;
+		uint8_t type;
+		uint8_t bus;
+	} entry[0];
+} __packed;
+
+#define MGMT_OP_READ_ADV_FEATURES	0x003D
+struct mgmt_rp_read_adv_features {
+	uint32_t supported_flags;
+	uint8_t  max_adv_data_len;
+	uint8_t  max_scan_rsp_len;
+	uint8_t  max_instances;
+	uint8_t  num_instances;
+	uint8_t  instance[0];
+} __packed;
+
+#define MGMT_OP_ADD_ADVERTISING		0x003E
+struct mgmt_cp_add_advertising {
+	uint8_t  instance;
+	uint32_t flags;
+	uint16_t duration;
+	uint16_t timeout;
+	uint8_t  adv_data_len;
+	uint8_t  scan_rsp_len;
+	uint8_t  data[0];
+} __packed;
+struct mgmt_rp_add_advertising {
+	uint8_t instance;
+} __packed;
+
+#define MGMT_ADV_FLAG_CONNECTABLE	(1 << 0)
+#define MGMT_ADV_FLAG_DISCOV		(1 << 1)
+#define MGMT_ADV_FLAG_LIMITED_DISCOV	(1 << 2)
+#define MGMT_ADV_FLAG_MANAGED_FLAGS	(1 << 3)
+#define MGMT_ADV_FLAG_TX_POWER		(1 << 4)
+#define MGMT_ADV_FLAG_APPEARANCE	(1 << 5)
+#define MGMT_ADV_FLAG_LOCAL_NAME	(1 << 6)
+
+#define MGMT_OP_REMOVE_ADVERTISING	0x003F
+struct mgmt_cp_remove_advertising {
+	uint8_t instance;
+} __packed;
+#define MGMT_REMOVE_ADVERTISING_SIZE	1
+struct mgmt_rp_remove_advertising {
+	uint8_t instance;
+} __packed;
+
+#define MGMT_OP_GET_ADV_SIZE_INFO	0x0040
+struct mgmt_cp_get_adv_size_info {
+	uint8_t  instance;
+	uint32_t flags;
+} __packed;
+#define MGMT_GET_ADV_SIZE_INFO_SIZE	5
+struct mgmt_rp_get_adv_size_info {
+	uint8_t  instance;
+	uint32_t flags;
+	uint8_t  max_adv_data_len;
+	uint8_t  max_scan_rsp_len;
+} __packed;
 
 #define MGMT_EV_CMD_COMPLETE		0x0001
 struct mgmt_ev_cmd_complete {
@@ -484,7 +554,7 @@ struct mgmt_ev_controller_error {
 
 #define MGMT_EV_CLASS_OF_DEV_CHANGED	0x0007
 struct mgmt_ev_class_of_dev_changed {
-	uint8_t class_of_dev[3];
+	uint8_t dev_class[3];
 } __packed;
 
 #define MGMT_EV_LOCAL_NAME_CHANGED	0x0008
@@ -641,6 +711,35 @@ struct mgmt_ev_new_conn_param {
 
 #define MGMT_EV_NEW_CONFIG_OPTIONS	0x001f
 
+#define MGMT_EV_EXT_INDEX_ADDED		0x0020
+struct mgmt_ev_ext_index_added {
+	uint8_t type;
+	uint8_t bus;
+} __packed;
+
+#define MGMT_EV_EXT_INDEX_REMOVED	0x0021
+struct mgmt_ev_ext_index_removed {
+	uint8_t type;
+	uint8_t bus;
+} __packed;
+
+#define MGMT_EV_LOCAL_OOB_DATA_UPDATED	0x0022
+struct mgmt_ev_local_oob_data_updated {
+	uint8_t  type;
+	uint16_t eir_len;
+	uint8_t  eir[0];
+} __packed;
+
+#define MGMT_EV_ADVERTISING_ADDED	0x0023
+struct mgmt_ev_advertising_added {
+	uint8_t instance;
+} __packed;
+
+#define MGMT_EV_ADVERTISING_REMOVED	0x0024
+struct mgmt_ev_advertising_removed {
+	uint8_t instance;
+} __packed;
+
 static const char *mgmt_op[] = {
 	"<0x0000>",
 	"Read Version",
@@ -650,7 +749,7 @@ static const char *mgmt_op[] = {
 	"Set Powered",
 	"Set Discoverable",
 	"Set Connectable",
-	"Set Fast Connectable",		/* 0x0008 */
+	"Set Fast Connectable",			/* 0x0008 */
 	"Set Bondable",
 	"Set Link Security",
 	"Set Secure Simple Pairing",
@@ -658,7 +757,7 @@ static const char *mgmt_op[] = {
 	"Set Low Energy",
 	"Set Dev Class",
 	"Set Local Name",
-	"Add UUID",			/* 0x0010 */
+	"Add UUID",					/* 0x0010 */
 	"Remove UUID",
 	"Load Link Keys",
 	"Load Long Term Keys",
@@ -666,7 +765,7 @@ static const char *mgmt_op[] = {
 	"Get Connections",
 	"PIN Code Reply",
 	"PIN Code Neg Reply",
-	"Set IO Capability",		/* 0x0018 */
+	"Set IO Capability",				/* 0x0018 */
 	"Pair Device",
 	"Cancel Pair Device",
 	"Unpair Device",
@@ -674,7 +773,7 @@ static const char *mgmt_op[] = {
 	"User Confirm Neg Reply",
 	"User Passkey Reply",
 	"User Passkey Neg Reply",
-	"Read Local OOB Data",		/* 0x0020 */
+	"Read Local OOB Data",				/* 0x0020 */
 	"Add Remote OOB Data",
 	"Remove Remove OOB Data",
 	"Start Discovery",
@@ -682,7 +781,7 @@ static const char *mgmt_op[] = {
 	"Confirm Name",
 	"Block Device",
 	"Unblock Device",
-	"Set Device ID",
+	"Set Device ID",				/* 0x0028 */
 	"Set Advertising",
 	"Set BR/EDR",
 	"Set Static Address",
@@ -690,7 +789,7 @@ static const char *mgmt_op[] = {
 	"Set Secure Connections",
 	"Set Debug Keys",
 	"Set Privacy",
-	"Load Identity Resolving Keys",
+	"Load Identity Resolving Keys",			/* 0x0030 */
 	"Get Connection Information",
 	"Get Clock Information",
 	"Add Device",
@@ -698,9 +797,16 @@ static const char *mgmt_op[] = {
 	"Load Connection Parameters",
 	"Read Unconfigured Index List",
 	"Read Controller Configuration Information",
-	"Set External Configuration",
+	"Set External Configuration",			/* 0x0038 */
 	"Set Public Address",
 	"Start Service Discovery",
+	"Read Local Out Of Band Extended Data",
+	"Read Extended Controller Index List",
+	"Read Advertising Features",
+	"Add Advertising",
+	"Remove Advertising",
+	"Get Advertising Size Information",		/* 0x0040 */
+	"Start Limited Discovery",
 };
 
 static const char *mgmt_ev[] = {
@@ -712,7 +818,7 @@ static const char *mgmt_ev[] = {
 	"Index Removed",
 	"New Settings",
 	"Class of Device Changed",
-	"Local Name Changed",		/* 0x0008 */
+	"Local Name Changed",				/* 0x0008 */
 	"New Link Key",
 	"New Long Term Key",
 	"Device Connected",
@@ -720,7 +826,7 @@ static const char *mgmt_ev[] = {
 	"Connect Failed",
 	"PIN Code Request",
 	"User Confirm Request",
-	"User Passkey Request",		/* 0x0010 */
+	"User Passkey Request",				/* 0x0010 */
 	"Authentication Failed",
 	"Device Found",
 	"Discovering",
@@ -728,7 +834,7 @@ static const char *mgmt_ev[] = {
 	"Device Unblocked",
 	"Device Unpaired",
 	"Passkey Notify",
-	"New Identity Resolving Key",
+	"New Identity Resolving Key",			/* 0x0018 */
 	"New Signature Resolving Key",
 	"Device Added",
 	"Device Removed",
@@ -736,6 +842,11 @@ static const char *mgmt_ev[] = {
 	"Unconfigured Index Added",
 	"Unconfigured Index Removed",
 	"New Configuration Options",
+	"Extended Index Added",				/* 0x0020 */
+	"Extended Index Removed",
+	"Local Out Of Band Extended Data Updated",
+	"Advertising Added",
+	"Advertising Removed",
 };
 
 static const char *mgmt_status[] = {
@@ -759,6 +870,7 @@ static const char *mgmt_status[] = {
 	"Invalid Index",
 	"Blocked through rfkill",
 	"Already Paired",
+	"Permission Denied",
 };
 
 #ifndef NELEM
