@@ -29,7 +29,10 @@
 #include <glib.h>
 #include <errno.h>
 
+#include "lib/bluetooth.h"
+#include "lib/sdp.h"
 #include "lib/uuid.h"
+
 #include "src/plugin.h"
 #include "src/adapter.h"
 #include "src/shared/util.h"
