@@ -43,6 +43,7 @@
 #include "gatt-database.h"
 #include "dbus-common.h"
 #include "profile.h"
+#include "service.h"
 
 #ifndef ATT_CID
 #define ATT_CID 4
@@ -230,15 +231,7 @@ static struct device_state *device_state_create(bdaddr_t *bdaddr,
 	struct device_state *dev_state;
 
 	dev_state = new0(struct device_state, 1);
-	if (!dev_state)
-		return NULL;
-
 	dev_state->ccc_states = queue_new();
-	if (!dev_state->ccc_states) {
-		free(dev_state);
-		return NULL;
-	}
-
 	bacpy(&dev_state->bdaddr, bdaddr);
 	dev_state->bdaddr_type = bdaddr_type;
 
@@ -260,8 +253,6 @@ static struct device_state *get_device_state(struct btd_gatt_database *database,
 		return dev_state;
 
 	dev_state = device_state_create(bdaddr, bdaddr_type);
-	if (!dev_state)
-		return NULL;
 
 	queue_push_tail(database->device_states, dev_state);
 
@@ -277,17 +268,12 @@ static struct ccc_state *get_ccc_state(struct btd_gatt_database *database,
 	struct ccc_state *ccc;
 
 	dev_state = get_device_state(database, bdaddr, bdaddr_type);
-	if (!dev_state)
-		return NULL;
 
 	ccc = find_ccc_state(dev_state, handle);
 	if (ccc)
 		return ccc;
 
 	ccc = new0(struct ccc_state, 1);
-	if (!ccc)
-		return NULL;
-
 	ccc->handle = handle;
 	queue_push_tail(dev_state->ccc_states, ccc);
 
@@ -383,6 +369,7 @@ static void profile_remove(void *data)
 	DBG("Removed \"%s\"", p->name);
 
 	adapter_foreach(adapter_remove_profile, p);
+	btd_profile_unregister(p);
 
 	g_free((void *) p->name);
 	g_free((void *) p->remote_uuid);
@@ -725,10 +712,6 @@ static void gatt_ccc_read_cb(struct gatt_db_attribute *attrib,
 	}
 
 	ccc = get_ccc_state(database, &bdaddr, bdaddr_type, handle);
-	if (!ccc) {
-		ecode = BT_ATT_ERROR_UNLIKELY;
-		goto done;
-	}
 
 	len = 2 - offset;
 	value = len ? &ccc->value[offset] : NULL;
@@ -771,10 +754,6 @@ static void gatt_ccc_write_cb(struct gatt_db_attribute *attrib,
 	}
 
 	ccc = get_ccc_state(database, &bdaddr, bdaddr_type, handle);
-	if (!ccc) {
-		ecode = BT_ATT_ERROR_UNLIKELY;
-		goto done;
-	}
 
 	ccc_cb = queue_find(database->ccc_callbacks, ccc_cb_match_handle,
 			UINT_TO_PTR(gatt_db_attribute_get_handle(attrib)));
@@ -811,10 +790,6 @@ service_add_ccc(struct gatt_db_attribute *service,
 	bt_uuid_t uuid;
 
 	ccc_cb = new0(struct ccc_cb_data, 1);
-	if (!ccc_cb) {
-		error("Could not allocate memory for callback data");
-		return NULL;
-	}
 
 	bt_uuid16_create(&uuid, GATT_CLIENT_CHARAC_CFG_UUID);
 	ccc = gatt_db_service_add_descriptor(service, &uuid,
@@ -1073,7 +1048,7 @@ static void client_disconnect_cb(DBusConnection *conn, void *user_data)
 	service_remove_helper(user_data);
 }
 
-static void service_remove(void *data)
+static void remove_service(void *data)
 {
 	struct external_service *service = data;
 
@@ -1100,21 +1075,8 @@ static struct external_chrc *chrc_create(struct external_service *service,
 	struct external_chrc *chrc;
 
 	chrc = new0(struct external_chrc, 1);
-	if (!chrc)
-		return NULL;
-
 	chrc->pending_reads = queue_new();
-	if (!chrc->pending_reads) {
-		free(chrc);
-		return NULL;
-	}
-
 	chrc->pending_writes = queue_new();
-	if (!chrc->pending_writes) {
-		queue_destroy(chrc->pending_reads, NULL);
-		free(chrc);
-		return NULL;
-	}
 
 	chrc->path = g_strdup(path);
 	if (!chrc->path) {
@@ -1137,21 +1099,8 @@ static struct external_desc *desc_create(struct external_service *service,
 	struct external_desc *desc;
 
 	desc = new0(struct external_desc, 1);
-	if (!desc)
-		return NULL;
-
 	desc->pending_reads = queue_new();
-	if (!desc->pending_reads) {
-		free(desc);
-		return NULL;
-	}
-
 	desc->pending_writes = queue_new();
-	if (!desc->pending_writes) {
-		queue_destroy(desc->pending_reads, NULL);
-		free(desc);
-		return NULL;
-	}
 
 	desc->chrc_path = g_strdup(chrc_path);
 	if (!desc->chrc_path) {
@@ -1452,7 +1401,7 @@ static void proxy_removed_cb(GDBusProxy *proxy, void *user_data)
 
 	DBG("Proxy removed - removing service: %s", service->path);
 
-	service_remove(service);
+	remove_service(service);
 }
 
 static bool parse_uuid(GDBusProxy *proxy, bt_uuid_t *uuid)
@@ -1598,8 +1547,6 @@ static struct pending_op *pending_read_new(struct queue *owner_queue,
 	struct pending_op *op;
 
 	op = new0(struct pending_op, 1);
-	if (!op)
-		return NULL;
 
 	op->owner_queue = owner_queue;
 	op->attrib = attrib;
@@ -1617,11 +1564,6 @@ static void send_read(struct gatt_db_attribute *attrib, GDBusProxy *proxy,
 	uint8_t ecode = BT_ATT_ERROR_UNLIKELY;
 
 	op = pending_read_new(owner_queue, attrib, id);
-	if (!op) {
-		error("Failed to allocate memory for pending read call");
-		ecode = BT_ATT_ERROR_INSUFFICIENT_RESOURCES;
-		goto error;
-	}
 
 	if (g_dbus_proxy_method_call(proxy, "ReadValue", NULL, read_reply_cb,
 						op, pending_op_free) == TRUE)
@@ -1629,7 +1571,6 @@ static void send_read(struct gatt_db_attribute *attrib, GDBusProxy *proxy,
 
 	pending_op_free(op);
 
-error:
 	gatt_db_attribute_read_result(attrib, id, ecode, NULL, 0);
 }
 
@@ -1689,8 +1630,6 @@ static struct pending_op *pending_write_new(struct queue *owner_queue,
 	struct pending_op *op;
 
 	op = new0(struct pending_op, 1);
-	if (!op)
-		return NULL;
 
 	op->data.iov_base = (uint8_t *) value;
 	op->data.iov_len = len;
@@ -1712,11 +1651,6 @@ static void send_write(struct gatt_db_attribute *attrib, GDBusProxy *proxy,
 	uint8_t ecode = BT_ATT_ERROR_UNLIKELY;
 
 	op = pending_write_new(owner_queue, attrib, id, value, len);
-	if (!op) {
-		error("Failed to allocate memory for pending read call");
-		ecode = BT_ATT_ERROR_INSUFFICIENT_RESOURCES;
-		goto error;
-	}
 
 	if (g_dbus_proxy_method_call(proxy, "WriteValue", write_setup_cb,
 						write_reply_cb, op,
@@ -1725,7 +1659,6 @@ static void send_write(struct gatt_db_attribute *attrib, GDBusProxy *proxy,
 
 	pending_op_free(op);
 
-error:
 	gatt_db_attribute_write_result(attrib, id, ecode);
 }
 
@@ -1783,22 +1716,16 @@ static uint8_t ccc_write_cb(uint16_t value, void *user_data)
 		return 0;
 	}
 
-	/*
-	 * TODO: All of the errors below should fall into the so called
-	 * "Application Error" range. Since there is no well defined error for
-	 * these, we return a generic ATT protocol error for now.
-	 */
-
 	if (chrc->ntfy_cnt == UINT_MAX) {
 		/* Maximum number of per-device CCC descriptors configured */
-		return BT_ATT_ERROR_REQUEST_NOT_SUPPORTED;
+		return BT_ATT_ERROR_INSUFFICIENT_RESOURCES;
 	}
 
 	/* Don't support undefined CCC values yet */
 	if (value > 2 ||
 		(value == 1 && !(chrc->props & BT_GATT_CHRC_PROP_NOTIFY)) ||
 		(value == 2 && !(chrc->props & BT_GATT_CHRC_PROP_INDICATE)))
-		return BT_ATT_ERROR_REQUEST_NOT_SUPPORTED;
+		return BT_ERROR_CCC_IMPROPERLY_CONFIGURED;
 
 	/*
 	 * Always call StartNotify for an incoming enable and ignore the return
@@ -1807,7 +1734,7 @@ static uint8_t ccc_write_cb(uint16_t value, void *user_data)
 	if (g_dbus_proxy_method_call(chrc->proxy,
 						"StartNotify", NULL, NULL,
 						NULL, NULL) == FALSE)
-		return BT_ATT_ERROR_REQUEST_NOT_SUPPORTED;
+		return BT_ATT_ERROR_UNLIKELY;
 
 	__sync_fetch_and_add(&chrc->ntfy_cnt, 1);
 
@@ -2147,10 +2074,10 @@ reply:
 	service->reg = NULL;
 
 	if (fail)
-		service_remove(service);
+		remove_service(service);
 }
 
-static struct external_service *service_create(DBusConnection *conn,
+static struct external_service *create_service(DBusConnection *conn,
 					DBusMessage *msg, const char *path)
 {
 	struct external_service *service;
@@ -2160,8 +2087,6 @@ static struct external_service *service_create(DBusConnection *conn,
 		return NULL;
 
 	service = new0(struct external_service, 1);
-	if (!service)
-		return NULL;
 
 	service->client = g_dbus_client_new_full(conn, sender, path, path);
 	if (!service->client)
@@ -2176,12 +2101,7 @@ static struct external_service *service_create(DBusConnection *conn,
 		goto fail;
 
 	service->chrcs = queue_new();
-	if (!service->chrcs)
-		goto fail;
-
 	service->descs = queue_new();
-	if (!service->descs)
-		goto fail;
 
 	service->reg = dbus_message_ref(msg);
 
@@ -2228,7 +2148,7 @@ static DBusMessage *manager_register_service(DBusConnection *conn,
 	if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_ARRAY)
 		return btd_error_invalid_args(msg);
 
-	service = service_create(conn, msg, path);
+	service = create_service(conn, msg, path);
 	if (!service)
 		return btd_error_failed(msg, "Failed to register service");
 
@@ -2284,28 +2204,47 @@ static void profile_exited(DBusConnection *conn, void *user_data)
 	profile_free(profile);
 }
 
+static int profile_device_probe(struct btd_service *service)
+{
+	struct btd_profile *p = btd_service_get_profile(service);
+
+	DBG("%s probed", p->name);
+
+	return 0;
+}
+
+static void profile_device_remove(struct btd_service *service)
+{
+	struct btd_profile *p = btd_service_get_profile(service);
+
+	DBG("%s removed", p->name);
+}
+
 static int profile_add(struct external_profile *profile, const char *uuid)
 {
 	struct btd_profile *p;
 
 	p = new0(struct btd_profile, 1);
-	if (!p)
-		return -ENOMEM;
 
 	/* Assign directly to avoid having extra fields */
 	p->name = (const void *) g_strdup_printf("%s%s/%s", profile->owner,
 							profile->path, uuid);
-	if (!p->name)
+	if (!p->name) {
+		free(p);
 		return -ENOMEM;
+	}
 
 	p->remote_uuid = (const void *) g_strdup(uuid);
 	if (!p->remote_uuid) {
 		g_free((void *) p->name);
-		free((void *) p);
+		free(p);
 		return -ENOMEM;
 	}
 
+	p->device_probe = profile_device_probe;
+	p->device_remove = profile_device_remove;
 	p->auto_connect = true;
+	p->external = true;
 
 	queue_push_tail(profile->profiles, p);
 
@@ -2318,6 +2257,7 @@ static void add_profile(void *data, void *user_data)
 {
 	struct btd_adapter *adapter = user_data;
 
+	btd_profile_register(data);
 	adapter_add_profile(adapter, data);
 }
 
@@ -2333,8 +2273,6 @@ static int profile_create(DBusConnection *conn,
 		return -EINVAL;
 
 	profile = new0(struct external_profile, 1);
-	if (!profile)
-		return -ENOMEM;
 
 	profile->owner = g_strdup(sender);
 	if (!profile->owner)
@@ -2345,9 +2283,6 @@ static int profile_create(DBusConnection *conn,
 		goto fail;
 
 	profile->profiles = queue_new();
-	if (!profile->profiles)
-		goto fail;
-
 	profile->database = database;
 	profile->id = g_dbus_add_disconnect_watch(conn, sender, profile_exited,
 								profile, NULL);
@@ -2480,29 +2415,12 @@ struct btd_gatt_database *btd_gatt_database_new(struct btd_adapter *adapter)
 		return NULL;
 
 	database = new0(struct btd_gatt_database, 1);
-	if (!database)
-		return NULL;
-
 	database->adapter = btd_adapter_ref(adapter);
 	database->db = gatt_db_new();
-	if (!database->db)
-		goto fail;
-
 	database->device_states = queue_new();
-	if (!database->device_states)
-		goto fail;
-
 	database->services = queue_new();
-	if (!database->services)
-		goto fail;
-
 	database->profiles = queue_new();
-	if (!database->profiles)
-		goto fail;
-
 	database->ccc_callbacks = queue_new();
-	if (!database->ccc_callbacks)
-		goto fail;
 
 	database->db_id = gatt_db_register(database->db, gatt_db_service_added,
 							gatt_db_service_removed,
