@@ -23,43 +23,39 @@
  *
  */
 
+typedef enum {
+	BT_MODE_DUAL,
+	BT_MODE_BREDR,
+	BT_MODE_LE,
+} bt_mode_t;
+
 struct main_opts {
-	char		host_name[40];
-	unsigned long	flags;
 	char		*name;
 	uint32_t	class;
-	uint16_t	pageto;
 	uint16_t	autoto;
-	uint32_t	discovto;
 	uint32_t	pairto;
-	uint16_t	link_mode;
-	uint16_t	link_policy;
-	gboolean	remember_powered;
+	uint32_t	discovto;
 	gboolean	reverse_sdp;
 	gboolean	name_resolv;
 	gboolean	debug_keys;
-	gboolean	attrib_server;
+	gboolean	fast_conn;
 
-	uint8_t		mode;
-	uint8_t		discov_interval;
-	char		deviceid[15]; /* FIXME: */
-};
+	uint16_t	did_source;
+	uint16_t	did_vendor;
+	uint16_t	did_product;
+	uint16_t	did_version;
 
-enum {
-	HCID_SET_NAME,
-	HCID_SET_CLASS,
-	HCID_SET_PAGETO,
-	HCID_SET_DISCOVTO,
+	bt_mode_t	mode;
 };
 
 extern struct main_opts main_opts;
 
-void btd_start_exit_timer(void);
-void btd_stop_exit_timer(void);
-
-gboolean plugin_init(GKeyFile *config, const char *enable,
-							const char *disable);
+gboolean plugin_init(const char *enable, const char *disable);
 void plugin_cleanup(void);
 
 void rfkill_init(void);
 void rfkill_exit(void);
+
+GKeyFile *btd_get_main_conf(void);
+
+void btd_exit(void);
