@@ -27,7 +27,7 @@
 #define __packed __attribute__((packed))
 #endif
 
-#define BT_ATT_SECURITY_NONE	0
+#define BT_ATT_SECURITY_AUTO	0
 #define BT_ATT_SECURITY_LOW	1
 #define BT_ATT_SECURITY_MEDIUM	2
 #define BT_ATT_SECURITY_HIGH	3
@@ -42,8 +42,8 @@
 #define BT_ATT_OP_MTU_RSP			0x03
 #define BT_ATT_OP_FIND_INFO_REQ			0x04
 #define BT_ATT_OP_FIND_INFO_RSP			0x05
-#define BT_ATT_OP_FIND_BY_TYPE_VAL_REQ		0x06
-#define BT_ATT_OP_FIND_BY_TYPE_VAL_RSP		0x07
+#define BT_ATT_OP_FIND_BY_TYPE_REQ		0x06
+#define BT_ATT_OP_FIND_BY_TYPE_RSP		0x07
 #define BT_ATT_OP_READ_BY_TYPE_REQ		0x08
 #define BT_ATT_OP_READ_BY_TYPE_RSP		0x09
 #define BT_ATT_OP_READ_REQ			0x0a
