@@ -114,7 +114,7 @@ static unsigned long disc_delay = 0;
 /* Initial sequence value when sending frames */
 static int seq_start = 0;
 
-static char *filename = NULL;
+static const char *filename = NULL;
 
 static int rfcmode = 0;
 static int master = 0;
@@ -592,6 +592,7 @@ static void do_listen(void (*handler)(int sk))
 
 	if (socktype == SOCK_DGRAM) {
 		handler(sk);
+		close(sk);
 		return;
 	}
 
@@ -644,7 +645,6 @@ static void do_listen(void (*handler)(int sk))
 			continue;
 		}
 		/* Child */
-		close(sk);
 
 		/* Set receive buffer size */
 		if (rcvbuf && setsockopt(nsk, SOL_SOCKET, SO_RCVBUF, &rcvbuf,
@@ -769,6 +769,7 @@ static void do_listen(void (*handler)(int sk))
 		}
 
 		handler(nsk);
+		close(sk);
 
 		syslog(LOG_INFO, "Disconnect: %m");
 		exit(0);
@@ -974,6 +975,8 @@ static void do_send(int sk)
 			sent += len;
 			size -= len;
 		}
+
+		close(fd);
 		return;
 	} else {
 		for (i = 6; i < data_size; i++)
@@ -1449,7 +1452,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'B':
-			filename = strdup(optarg);
+			filename = optarg;
 			break;
 
 		case 'N':
