@@ -33,13 +33,14 @@
 #include <signal.h>
 #include <sys/signalfd.h>
 #include <inttypes.h>
+#include <wordexp.h>
 
 #include <readline/readline.h>
 #include <readline/history.h>
 #include <glib.h>
-#include <gdbus.h>
 
-#include <client/display.h>
+#include "gdbus/gdbus.h"
+#include "client/display.h"
 
 /* String display constants */
 #define COLORED_NEW	COLOR_GREEN "NEW" COLOR_OFF
@@ -757,7 +758,7 @@ static void send_reply(DBusMessage *message, void *user_data)
 	dbus_error_init(&error);
 
 	if (dbus_set_error_from_message(&error, message) == TRUE) {
-		rl_printf("Failed to send: %s\n", error.name);
+		rl_printf("Failed to send/pull: %s\n", error.name);
 		dbus_error_free(&error);
 		return;
 	}
@@ -791,6 +792,23 @@ static void opp_send(GDBusProxy *proxy, int argc, char *argv[])
 						g_dbus_proxy_get_path(proxy));
 }
 
+static void opp_pull(GDBusProxy *proxy, int argc, char *argv[])
+{
+	if (argc < 2) {
+		rl_printf("Missing file argument\n");
+		return;
+	}
+
+	if (g_dbus_proxy_method_call(proxy, "PullBusinessCard", send_setup,
+			send_reply, g_strdup(argv[1]), g_free) == FALSE) {
+		rl_printf("Failed to pull\n");
+		return;
+	}
+
+	rl_printf("Attempting to pull %s from %s\n", argv[1],
+						g_dbus_proxy_get_path(proxy));
+}
+
 static void push_reply(DBusMessage *message, void *user_data)
 {
 	DBusMessageIter iter;
@@ -868,6 +886,22 @@ static void cmd_send(int argc, char *argv[])
 	rl_printf("Command not supported\n");
 }
 
+static void cmd_pull(int argc, char *argv[])
+{
+	GDBusProxy *proxy;
+
+	if (!check_default_session())
+		return;
+
+	proxy = find_opp(g_dbus_proxy_get_path(default_session));
+	if (proxy) {
+		opp_pull(proxy, argc, argv);
+		return;
+	}
+
+	rl_printf("Command not supported\n");
+}
+
 static void change_folder_reply(DBusMessage *message, void *user_data)
 {
 	DBusError error;
@@ -1232,7 +1266,6 @@ static void list_messages_reply(DBusMessage *message, void *user_data)
 {
 	DBusError error;
 	DBusMessageIter iter, array;
-	int ctype;
 
 	dbus_error_init(&error);
 
@@ -1249,8 +1282,8 @@ static void list_messages_reply(DBusMessage *message, void *user_data)
 
 	dbus_message_iter_recurse(&iter, &array);
 
-	while ((ctype = dbus_message_iter_get_arg_type(&array)) ==
-							DBUS_TYPE_DICT_ENTRY) {
+	while ((dbus_message_iter_get_arg_type(&array)) ==
+						DBUS_TYPE_DICT_ENTRY) {
 		DBusMessageIter entry;
 		const char *obj;
 
@@ -1281,13 +1314,8 @@ static void list_messages_setup(DBusMessageIter *iter, void *user_data)
 	dbus_message_iter_close_container(iter, &dict);
 }
 
-static void pbap_ls(GDBusProxy *proxy, int argc, char *argv[])
+static void pbap_list(GDBusProxy *proxy, int argc, char *argv[])
 {
-	if (argc > 1) {
-		pbap_search(proxy, argc, argv);
-		return;
-	}
-
 	if (g_dbus_proxy_method_call(proxy, "List", list_setup, list_reply,
 						NULL, NULL) == FALSE) {
 		rl_printf("Failed to List\n");
@@ -1297,6 +1325,51 @@ static void pbap_ls(GDBusProxy *proxy, int argc, char *argv[])
 	rl_printf("Attempting to List\n");
 }
 
+static void get_size_reply(DBusMessage *message, void *user_data)
+{
+	GDBusProxy *proxy = user_data;
+	DBusError error;
+	DBusMessageIter iter;
+
+	dbus_error_init(&error);
+
+	if (dbus_set_error_from_message(&error, message) == TRUE) {
+		rl_printf("Failed to GetSize: %s\n", error.name);
+		dbus_error_free(&error);
+		return;
+	}
+
+	dbus_message_iter_init(message, &iter);
+
+	print_iter("\t", "Size", &iter);
+
+	pbap_list(proxy, 0, NULL);
+}
+
+static void pbap_get_size(GDBusProxy *proxy, int argc, char *argv[])
+{
+	if (g_dbus_proxy_method_call(proxy, "GetSize", NULL, get_size_reply,
+						proxy, NULL) == FALSE) {
+		rl_printf("Failed to GetSize\n");
+		return;
+	}
+
+	rl_printf("Attempting to GetSize\n");
+}
+
+static void pbap_ls(GDBusProxy *proxy, int argc, char *argv[])
+{
+	if (argc > 1) {
+		if (strcmp("-l", argv[1]))
+			pbap_search(proxy, argc, argv);
+		else
+			pbap_get_size(proxy, argc, argv);
+		return;
+	}
+
+	pbap_list(proxy, argc, argv);
+}
+
 static void map_ls_messages(GDBusProxy *proxy, int argc, char *argv[])
 {
 	if (g_dbus_proxy_method_call(proxy, "ListMessages", list_messages_setup,
@@ -1939,8 +2012,10 @@ static const struct {
 	{ "suspend",      "<transfer>", cmd_suspend, "Suspend transfer" },
 	{ "resume",       "<transfer>", cmd_resume, "Resume transfer" },
 	{ "send",         "<file>",   cmd_send, "Send file" },
+	{ "pull",	  "<file>",   cmd_pull,
+					"Pull Vobject & stores in file" },
 	{ "cd",           "<path>",   cmd_cd, "Change current folder" },
-	{ "ls",           NULL,       cmd_ls, "List current folder" },
+	{ "ls",           "<options>", cmd_ls, "List current folder" },
 	{ "cp",          "<source file> <destination file>",   cmd_cp,
 				"Copy source file to destination file" },
 	{ "mv",          "<source file> <destination file>",   cmd_mv,
@@ -1990,8 +2065,9 @@ static char **cmd_completion(const char *text, int start, int end)
 
 static void rl_handler(char *input)
 {
+	wordexp_t w;
 	int argc;
-	char **argv = NULL;
+	char **argv;
 	int i;
 
 	if (!input) {
@@ -2005,18 +2081,16 @@ static void rl_handler(char *input)
 	if (!strlen(input))
 		goto done;
 
-	g_strstrip(input);
 	add_history(input);
 
-	argv = g_strsplit(input, " ", -1);
-	if (argv == NULL)
+	if (wordexp(input, &w, WRDE_NOCMD))
 		goto done;
 
-	for (argc = 0; argv[argc];)
-		argc++;
+	if (w.we_wordc == 0)
+		goto free_we;
 
-	if (argc == 0)
-		goto done;
+	argv = w.we_wordv;
+	argc = w.we_wordc;
 
 	for (i = 0; cmd_table[i].cmd; i++) {
 		if (strcmp(argv[0], cmd_table[i].cmd))
@@ -2024,13 +2098,13 @@ static void rl_handler(char *input)
 
 		if (cmd_table[i].func) {
 			cmd_table[i].func(argc, argv);
-			goto done;
+			goto free_we;
 		}
 	}
 
 	if (strcmp(argv[0], "help")) {
 		printf("Invalid command\n");
-		goto done;
+		goto free_we;
 	}
 
 	printf("Available commands:\n");
@@ -2043,8 +2117,9 @@ static void rl_handler(char *input)
 					cmd_table[i].desc ? : "");
 	}
 
+free_we:
+	wordfree(&w);
 done:
-	g_strfreev(argv);
 	free(input);
 }
 
